from django.contrib.auth.signals import user_logged_out
from django.db import models
from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _

from drf_audit_trail.managers import LoginAuditEventManager
from drf_audit_trail.mixins import BaseModelMixin

from .request_audit_event import RequestAuditEvent


class LoginAuditEvent(BaseModelMixin):
    """
    This is the model for the login audit trail.
    """

    SIGNIN = "Sign in"
    SIGNOUT = "Sign out"
    FAILED = "Failed"
    STATUS = (
        (SIGNIN, _("Sign in")),
        (SIGNOUT, _("Sign out")),
        (FAILED, _("Failed")),
    )

    status = models.CharField(_("Status"), choices=STATUS, max_length=8)
    request = models.OneToOneField(RequestAuditEvent, on_delete=models.CASCADE)

    objects: LoginAuditEventManager = LoginAuditEventManager()

    class Meta:
        verbose_name = _("Login audit event")
        verbose_name_plural = _("Login audit events")


@receiver(user_logged_out)
def audit_user_logout(sender, request, user, **kwargs):
    drf_login_audit_event = request.META.get("drf_login_audit_event")
    drf_request_audit_event = request.META.get("drf_request_audit_event")

    if drf_login_audit_event is not None:
        drf_login_audit_event["status"] = LoginAuditEvent.SIGNOUT
    if drf_request_audit_event is not None:
        drf_request_audit_event["user"] = user



# Generated by Django 4.2.6 on 2024-09-09 00:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('drf_audit_trail', '0003_alter_requestauditevent_user'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProcessAuditEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_informations', models.TextField(blank=True, null=True, verbose_name='Extra information')),
                ('datetime', models.DateTimeField(auto_now_add=True, verbose_name='Datetime')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('created_by', models.CharField(blank=True, max_length=120, null=True, verbose_name='User identifier')),
                ('request', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='processes', to='drf_audit_trail.requestauditevent', verbose_name='Request')),
            ],
            options={
                'verbose_name': 'Process audit event',
                'verbose_name_plural': 'Process audit events',
                'db_table': 'process_audit_event',
            },
        ),
        migrations.CreateModel(
            name='StepAuditEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_informations', models.TextField(blank=True, null=True, verbose_name='Extra information')),
                ('datetime', models.DateTimeField(auto_now_add=True, verbose_name='Datetime')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('order', models.IntegerField()),
                ('total_registrations', models.IntegerField(default=1)),
                ('created_by', models.CharField(blank=True, max_length=120, null=True, verbose_name='User identifier')),
                ('process', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='steps', to='drf_audit_trail.processauditevent')),
            ],
            options={
                'verbose_name': 'Step audit event',
                'verbose_name_plural': 'Step audit events',
                'db_table': 'step_audit_event',
            },
        ),
        migrations.CreateModel(
            name='RegistrationAuditEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra_informations', models.TextField(blank=True, null=True, verbose_name='Extra information')),
                ('datetime', models.DateTimeField(auto_now_add=True, verbose_name='Datetime')),
                ('success', models.BooleanField()),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
                ('created_by', models.CharField(blank=True, max_length=120, null=True, verbose_name='User identifier')),
                ('step', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='registrations', to='drf_audit_trail.stepauditevent')),
            ],
            options={
                'verbose_name': 'Registration audit event',
                'verbose_name_plural': 'Registration audit events',
                'db_table': 'registration_audit_event',
            },
        ),
    ]



from time import time

from django.db import transaction
from django.db.models import Manager

from drf_audit_trail.settings import DRF_AUDIT_TRAIL_USER_PK_NAME
from drf_audit_trail.utils import (
    get_authenticated_user_by_request,
    get_extra_informations,
    get_ip_addresses,
    get_response_size,
)


class RequestAuditEventManager(Manager):
    def create_by_request(self, request, response=None):
        request_meta = getattr(request, "META")
        start_time = request_meta.get("drf_audit_trail_request_start_time")
        authenticated_user = get_authenticated_user_by_request(request)

        drf_request_audit_event = request_meta.get("drf_request_audit_event", {})

        error_type = drf_request_audit_event.get("error_type")
        error_message = drf_request_audit_event.get("error_message")
        error_stacktrace = drf_request_audit_event.get("error_stacktrace")
        extra_informations = get_extra_informations(drf_request_audit_event)

        process_audit_event = request_meta.get("process_audit_event")

        instance = None
        with transaction.atomic():
            instance = self.create(
                url=request.path,
                method=request.method,
                query_params=request.META.get("QUERY_STRING"),
                ip_addresses=get_ip_addresses(request),
                request_type=request.scheme,
                user=getattr(authenticated_user, DRF_AUDIT_TRAIL_USER_PK_NAME, None),
                response_time=time() - start_time,
                response_size=get_response_size(response),
                status_code=getattr(response, "status_code", None),
                error_type=error_type,
                error_message=error_message,
                error_stacktrace=error_stacktrace,
                extra_informations=extra_informations,
            )

        if process_audit_event is not None and instance is not None:
            try:
                instance.processes.add(process_audit_event)
            except BaseException:
                pass

        return instance



import re
from time import time
from traceback import format_exc as traceback_format_exc

from asgiref.sync import sync_to_async
from django.http import HttpRequest
from django.utils.deprecation import MiddlewareMixin

from drf_audit_trail.models import LoginAuditEvent, RequestAuditEvent
from drf_audit_trail.settings import (
    DRF_AUDIT_TRAIL_AUTH_URL,
    DRF_AUDIT_TRAIL_REQUEST_AUDIT_URLS,
)


class RequestLoginAuditEventMiddleware(MiddlewareMixin):
    def _start_request(self, request):
        request_audit_event_enabled = self._is_request_audit_enabled(request)

        request.META["drf_request_audit_event"] = {}
        request.META["drf_login_audit_event"] = {}
        request.META["drf_audit_trail_request_start_time"] = time()

        return request_audit_event_enabled

    async def __acall__(self, request):
        request_audit_event_enabled = self._start_request(request)

        response = await self.get_response(request)
        if request_audit_event_enabled:
            await sync_to_async(self._create_instances, thread_sensitive=True)(
                request, response
            )

        return response

    def process_request(self, request: HttpRequest):
        request_audit_event_enabled = self._start_request(request)

        response = self.get_response(request)
        if request_audit_event_enabled:
            self._create_instances(request, response)

        return response

    def process_exception(self, request, exception: BaseException):
        request.META["drf_request_audit_event"] = {
            "error_type": exception.__class__.__name__,
            "error_message": str(exception),
            "error_stacktrace": traceback_format_exc(),
        }

        if self._is_auth_url(request):
            request.META["drf_login_audit_event"]["status"] = LoginAuditEvent.FAILED

    def _create_instances(self, request, response):
        request_audit_event = RequestAuditEvent.objects.create_by_request(
            request, response
        )

        if self._is_auth_url(request) and isinstance(
            request_audit_event, RequestAuditEvent
        ):
            LoginAuditEvent.objects.create_by_request(
                request, response, request_audit_event
            )

        return response

    def _is_request_audit_enabled(self, request):
        for i in DRF_AUDIT_TRAIL_REQUEST_AUDIT_URLS:
            if bool(re.match(i, request.path)):
                return True

        return self._is_auth_url(request)

    def _is_auth_url(self, request):
        if request.method != "POST":
            return False

        def validate(auth_url):
            return auth_url == request.META.get("PATH_INFO")

        if isinstance(DRF_AUDIT_TRAIL_AUTH_URL, list):
            return any(validate(auth_url) for auth_url in DRF_AUDIT_TRAIL_AUTH_URL)

        return validate(DRF_AUDIT_TRAIL_AUTH_URL)



from rest_framework.response import Response
from rest_framework.views import APIView

from drf_audit_trail.models import (
    LoginAuditEvent,
)


class LogoutAPIView(APIView):
    def post(self, request, *args, **kwargs):
        drf_login_audit_event = request.META.get("drf_login_audit_event")
        if drf_login_audit_event is not None:
            drf_login_audit_event["status"] = LoginAuditEvent.SIGNOUT
        return Response({"detail": "Successfully logged out."})


logout_api_view = LogoutAPIView.as_view()



from django.db import transaction
from django.db.models import Manager

from drf_audit_trail.settings import DRF_AUDIT_TRAIL_AUTH_STATUS_CODE_FALIED
from drf_audit_trail.utils import get_extra_informations


class LoginAuditEventManager(Manager):
    def create_by_request(self, request, response, request_audit_event):
        request_meta = getattr(request, "META")
        drf_login_audit_event = request_meta.get("drf_login_audit_event", {})
        status = None

        if response.status_code == DRF_AUDIT_TRAIL_AUTH_STATUS_CODE_FALIED:
            status = "Failed"
        elif not drf_login_audit_event.get("status"):
            status = "Sign in" if request.user.is_authenticated else "Failed"
        else:
            status = drf_login_audit_event.get("status")

        extra_informations = get_extra_informations(drf_login_audit_event)

        with transaction.atomic():
            return self.create(
                status=status,
                request=request_audit_event,
                extra_informations=extra_informations,
            )



from django.contrib.auth.models import User
from django.test import Client, TestCase

from drf_audit_trail.models import LoginAuditEvent, RequestAuditEvent


class MiddlewareTestCase(TestCase):
    databases = {"default", "audit_trail"}

    def _get_client_with_login(self):
        user_data = {"username": "talismar", "password": "admin"}
        User.objects.create_user(**user_data)

        client = Client()
        response = client.post("/api/token/", data=user_data)
        token = response.json()["access"]

        return Client(HTTP_Authorization="Bearer " + token)

    def test_should_store_a_login_audit_event_for_auth_request(self):
        User.objects.create_user(username="talismar", password="admin")

        client = Client()

        request_data = {"username": "talismar", "password": "admin"}
        client.post("/api/token/", data=request_data)

        login_audit_event = LoginAuditEvent.objects.all()

        self.assertEqual(login_audit_event.count(), 1)
        self.assertEqual(login_audit_event[0].status, LoginAuditEvent.SIGNIN)

    def test_should_store_a_login_audit_event_for_auth_request_with_falied_status(
        self,
    ):
        client = Client()

        request_data = {"username": "talismar", "password": "admin"}
        client.post("/api/token/", data=request_data)

        login_audit_event = LoginAuditEvent.objects.all()

        self.assertEqual(login_audit_event.count(), 1)
        self.assertEqual(login_audit_event[0].status, LoginAuditEvent.FAILED)

    def test_should_store_a_request_audit_event_for_any_request_not_found_that_request_url_match(
        self,
    ):
        client = Client()

        client.get("/api/fake-endpoint/")

        request_audit_event = RequestAuditEvent.objects.all()

        self.assertEqual(request_audit_event.count(), 1)
        self.assertEqual(request_audit_event[0].method, "GET")
        self.assertEqual(request_audit_event[0].status_code, 404)
        self.assertEqual(request_audit_event[0].query_params, "")
        self.assertIsNone(request_audit_event[0].user)

    def test_should_store_the_request_user_for_protected_endpoint_or_when_user_is_authenticated(
        self,
    ):
        client = self._get_client_with_login()

        response = client.get("/api/protected-endpoint/")

        request_audit_event = RequestAuditEvent.objects.filter(
            url="/api/protected-endpoint/"
        ).first()

        self.assertIsNotNone(request_audit_event.user)
        self.assertEqual(request_audit_event.user, "1")
        self.assertEqual(response.status_code, request_audit_event.status_code)



from django.contrib import admin
from django.contrib.auth import get_user_model

from .models import (
    LoginAuditEvent,
    ProcessAuditEvent,
    RegistrationAuditEvent,
    RequestAuditEvent,
    StepAuditEvent,
)
from .settings import DRF_AUDIT_TRAIL_USER_PK_NAME

UserModel = get_user_model()


def _get_user_by_id(user_id: str | None):
    try:
        filter_param = {DRF_AUDIT_TRAIL_USER_PK_NAME: user_id}
        user = UserModel.objects.filter(**filter_param).first()
        if user is not None:
            return user
    except ValueError:
        pass


class RequestAuditEventModelAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "method",
        "url",
        "status_code",
        "_user",
        "datetime",
        "request_type",
    )
    list_filter = ("method", "ip_addresses")
    search_fields = ("method", "ip_addresses", "status_code")

    def _user(self, obj: RequestAuditEvent):
        return _get_user_by_id(obj.user)


admin.site.register(RequestAuditEvent, RequestAuditEventModelAdmin)


class LoginAuditEventModelAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "user",
        "status",
        "datetime",
        "request_ip_addresses",
        "request__status_code",
    )
    list_filter = ("status",)

    @admin.display()
    def request_ip_addresses(self, obj):
        if obj.request is not None:
            return obj.request.ip_addresses

    @admin.display()
    def request__status_code(self, obj):
        if obj.request is not None:
            return obj.request.status_code

    def user(self, obj):
        if obj.request is not None:
            return _get_user_by_id(obj.request.user)


admin.site.register(LoginAuditEvent, LoginAuditEventModelAdmin)


class ProcessAuditEventModelAdmin(admin.ModelAdmin):
    list_display = ("id", "name", "success", "request", "user")
    list_filter = ("name",)
    search_fields = ("name",)

    def user(self, obj):
        return _get_user_by_id(obj.created_by)

    @admin.display(boolean=True)
    def success(self, obj):
        if obj.steps.filter(registrations__success=False).exists():
            return False
        return True


admin.site.register(ProcessAuditEvent, ProcessAuditEventModelAdmin)


class StepAuditEventModelAdmin(admin.ModelAdmin):
    list_display = ("id", "name", "success", "process", "user")
    list_filter = ("name",)
    search_fields = ("name",)

    def user(self, obj):
        return _get_user_by_id(obj.created_by)

    @admin.display(boolean=True)
    def success(self, obj):
        if obj.registrations.filter(success=True).count() == obj.total_registrations:
            return True
        return False


admin.site.register(StepAuditEvent, StepAuditEventModelAdmin)


class RegistrationAuditEventModelAdmin(admin.ModelAdmin):
    list_display = ("id", "name", "success", "step", "user")
    list_filter = ("success",)
    search_fields = ("message",)

    # @admin.display()
    # def step_name(self, obj):
    #     if obj.step is not None:
    #         return obj.step.name

    def user(self, obj):
        return _get_user_by_id(obj.created_by)


admin.site.register(RegistrationAuditEvent, RegistrationAuditEventModelAdmin)



from django.http import HttpRequest

from drf_audit_trail.models import ProcessAuditEvent


class ProcessAudit:
    def __init__(self, request: HttpRequest):
        self.request = request
        self._process = self.create_process()
        if not isinstance(self._process, ProcessAuditEvent):
            raise TypeError("self._process should be type ProcessAuditEvent")

        request.META["process_audit_event"] = self._process

        self.create_steps(self._process)

    def save_model(self, model):
        if self.request.user.is_authenticated:
            model.created_by = self.request.user.pk
        model.save()
        return model

    def create_process(self) -> ProcessAuditEvent:
        raise NotImplementedError()

    def create_steps(self, process: ProcessAuditEvent):
        pass



# Generated by Django 5.0.7 on 2024-08-05 22:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("drf_audit_trail", "0002_alter_loginauditevent_request"),
    ]

    operations = [
        migrations.AlterField(
            model_name="requestauditevent",
            name="user",
            field=models.CharField(
                blank=True, max_length=120, null=True, verbose_name="User identifier"
            ),
        ),
    ]



from drf_audit_trail.settings import DRF_AUDIT_TRAIL_DATABASE_ALIAS


class DRFAuditTrail:
    """
    A router to control all database operations on models in the
    drf_audit_trail application.
    """

    route_app_labels = {"drf_audit_trail"}

    def db_for_read(self, model, **hints):
        if model._meta.app_label in self.route_app_labels:
            return DRF_AUDIT_TRAIL_DATABASE_ALIAS
        return None

    def db_for_write(self, model, **hints):
        if model._meta.app_label in self.route_app_labels:
            return DRF_AUDIT_TRAIL_DATABASE_ALIAS
        return None

    def allow_relation(self, obj1, obj2, **hints):
        if obj1._meta.app_label in self.route_app_labels or (
            obj2._meta.app_label in self.route_app_labels
        ):
            return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if app_label in self.route_app_labels:
            return db == DRF_AUDIT_TRAIL_DATABASE_ALIAS

        return None



from django.db import models
from django.utils.translation import gettext_lazy as _

from drf_audit_trail.mixins import BaseModelMixin

from .request_audit_event import RequestAuditEvent


class ProcessAuditEvent(BaseModelMixin):
    """
    This is the model for the process audit trail.
    """

    name = models.CharField(_("Name"), max_length=255)
    description = models.TextField(_("Description"), null=True, blank=True)
    created_by = models.CharField(
        _("User identifier"), null=True, blank=True, max_length=120
    )

    request = models.ForeignKey(
        RequestAuditEvent,
        on_delete=models.CASCADE,
        verbose_name=_("Request"),
        null=True,
        blank=True,
        related_name="processes",
    )

    def register_step(self, **kwargs):
        pass

    def __str__(self) -> str:
        return "ProcessAuditEvent: %s" % self.pk

    class Meta:
        verbose_name = _("Process audit event")
        verbose_name_plural = _("Process audit events")


class StepAuditEvent(BaseModelMixin):
    process = models.ForeignKey(
        ProcessAuditEvent, on_delete=models.CASCADE, related_name="steps"
    )
    name = models.CharField(max_length=255)
    description = models.TextField(_("Description"), null=True, blank=True)
    order = models.IntegerField()
    total_registrations = models.IntegerField(default=1)
    created_by = models.CharField(
        _("User identifier"), null=True, blank=True, max_length=120
    )

    def __str__(self) -> str:
        return "StepAuditEvent: %s" % self.pk

    class Meta:
        verbose_name = _("Step audit event")
        verbose_name_plural = _("Step audit events")


class RegistrationAuditEvent(BaseModelMixin):
    step = models.ForeignKey(
        StepAuditEvent, on_delete=models.CASCADE, related_name="registrations"
    )
    success = models.BooleanField()
    name = models.CharField(max_length=255)
    description = models.TextField(blank=True, null=True)
    created_by = models.CharField(
        _("User identifier"), null=True, blank=True, max_length=120
    )

    def __str__(self) -> str:
        return "RegistrationAuditEvent: %s" % self.pk

    class Meta:
        verbose_name = _("Registration audit event")
        verbose_name_plural = _("Registration audit events")



from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK
from rest_framework_simplejwt.exceptions import InvalidToken, TokenError
from rest_framework_simplejwt.views import TokenObtainPairView

from drf_audit_trail.models import LoginAuditEvent


class DRFAuditTrailIntegrationMixin:
    def post(self, request, *args: tuple, **kwargs: dict):
        serializer = self.get_serializer(data=request.data)
        drf_login_audit_event = request.META.get("drf_login_audit_event")
        drf_request_audit_event = request.META.get("drf_request_audit_event")

        try:
            serializer.is_valid(raise_exception=True)
            drf_login_audit_event["status"] = LoginAuditEvent.SIGNIN
            drf_request_audit_event["user"] = serializer.user
        except TokenError as e:
            raise InvalidToken(e.args[0])

        return Response(serializer.validated_data, status=HTTP_200_OK)


class DRFAuditTrailTokenObtainPairView(
    DRFAuditTrailIntegrationMixin, TokenObtainPairView
):
    pass



from .login_audit_event import LoginAuditEvent
from .process_audit_event import (
    ProcessAuditEvent,
    RegistrationAuditEvent,
    StepAuditEvent,
)
from .request_audit_event import RequestAuditEvent

__all__ = [
    "LoginAuditEvent",
    "RequestAuditEvent",
    "ProcessAuditEvent",
    "RegistrationAuditEvent",
    "StepAuditEvent",
]



from drf_audit_trail.models import ProcessAuditEvent
from django.http import HttpResponse
from weasyprint import HTML
from django.template.loader import get_template


def process_report_view(request):
    processes = ProcessAuditEvent.objects.prefetch_related("steps__registrations").all()

    template = get_template("report_pdf.html")
    html_content = template.render({"processes": processes})

    pdf_file = HTML(string=html_content).write_pdf()

    response = HttpResponse(pdf_file, content_type="application/pdf")
    response["Content-Disposition"] = 'inline; filename="process_report.pdf"'
    return response



from rest_framework.authtoken.models import Token
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.response import Response

from drf_audit_trail.models import LoginAuditEvent


class DRFAuditTrailTokenAuthentication(ObtainAuthToken):
    def post(self, request, *args, **kwargs):
        drf_login_audit_event = request.META.get("drf_login_audit_event")
        drf_request_audit_event = request.META.get("drf_request_audit_event")

        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data["user"]

        drf_login_audit_event["status"] = LoginAuditEvent.SIGNIN
        drf_request_audit_event["user"] = user

        token, created = Token.objects.get_or_create(user=user)
        return Response({"token": token.key})



from django.db import models
from django.utils.translation import gettext_lazy as _

from drf_audit_trail.managers import RequestAuditEventManager
from drf_audit_trail.mixins import BaseModelMixin


class RequestAuditEvent(BaseModelMixin):
    """
    This is the model for the request audit trail.
    """

    user = models.CharField(_("User identifier"), null=True, blank=True, max_length=120)

    ip_addresses = models.CharField(_("Ip addresses"), max_length=50, null=True)

    """
        methods: GET, POST, PUT, DELETE, PATCH, OPTIONS
    """
    method = models.CharField(_("Method"), max_length=10)
    url = models.CharField(_("URL"), null=False, max_length=2048)

    # Request Parameters
    query_params = models.CharField(
        _("Query Parameters"), blank=True, null=True, max_length=2048
    )
    request_type = models.CharField(
        _("Request Type"), max_length=10, null=True, blank=True
    )
    # body = models.TextField(_("Body"), blank=True, default=dict)

    # Response Information
    status_code = models.IntegerField(_("Status Code"), null=True)
    response_time = models.FloatField(_("Response Time (ms)"), null=True)
    response_size = models.IntegerField(_("Response Size (bytes)"), null=True)

    # Error Handling
    error_type = models.CharField(
        _("Error Type"), max_length=255, blank=True, null=True
    )
    error_message = models.TextField(_("Error Message"), blank=True, null=True)
    error_stacktrace = models.TextField(_("Error Stacktrace"), blank=True, null=True)

    objects: RequestAuditEventManager = RequestAuditEventManager()

    def __str__(self) -> str:
        return "RequestAuditEvent: %s" % self.pk

    class Meta:
        verbose_name = _("Request event")
        verbose_name_plural = _("Request events")



from django import template
from django.contrib.auth import get_user_model

from drf_audit_trail.settings import DRF_AUDIT_TRAIL_USER_PK_NAME

UserModel = get_user_model()


register = template.Library()


@register.filter(name="range")
def make_range(value):
    return range(value)


@register.filter()
def get_item_at_index(list_obj, index):
    try:
        return list_obj[index]
    except IndexError:
        return None


@register.filter(name="get_user_by_id")
def get_user_by_id(user_id):
    try:
        filter_param = {DRF_AUDIT_TRAIL_USER_PK_NAME: user_id}
        user = UserModel.objects.filter(**filter_param).first()
        if user is not None:
            return user
    except ValueError:
        return "-"



from django.db import models
from django.utils.translation import gettext_lazy as _


class BaseModelMixin(models.Model):
    extra_informations = models.TextField(_("Extra information"), null=True, blank=True)
    datetime = models.DateTimeField(verbose_name=_("Datetime"), auto_now_add=True)

    class Meta:
        abstract = True
        ordering = ["-datetime"]

    def __str__(self) -> str:
        return "Id: %s" % self.pk



import json

from django.contrib.auth import get_user_model
from rest_framework_simplejwt.tokens import AccessToken, TokenError

User = get_user_model()


def _get_remote_addr(environ: dict):
    return environ.get("HTTP_X_FORWARDED_FOR") or environ.get("REMOTE_ADDR")


def get_ip_addresses(request):
    if isinstance(request, dict):
        return _get_remote_addr(request)

    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        ip = x_forwarded_for.split(",")[0]
    else:
        ip = request.META.get("REMOTE_ADDR")
    return ip


def get_response_size(response):
    content = getattr(response, "content", None)
    if content is not None and type(content) is bytes:
        return len(response.content)


def get_user_by_raw_authorization_header(request):
    authorization_header = request.headers.get("Authorization")

    if not authorization_header or "Bearer " not in authorization_header:
        return None

    try:
        raw_access_token = authorization_header.split("Bearer ")[1]
        access_token = AccessToken(raw_access_token)
        return User.objects.get(pk=access_token.get("user_id"))
    except (TokenError, User.DoesNotExist, IndexError):
        return None


def get_authenticated_user_by_request(request):
    user = request.user if request.user.is_authenticated else None
    if not user:
        user = request.META.get("drf_request_audit_event", {}).get("user")
    if not user:
        user = get_user_by_raw_authorization_header(request)

    return user


def is_json(data):
    try:
        json.loads(data)
    except BaseException:
        return False
    return True


def get_extra_informations(drf_request_audit_event: dict | None):
    if drf_request_audit_event is None:
        return None

    extra_informations = drf_request_audit_event.get("extra_informations")
    if extra_informations is not None and is_json(extra_informations):
        return extra_informations

    try:
        return json.dumps(extra_informations)
    except Exception:
        return None



from django.conf import settings

# Retention Period
"""
Refere-se a quanitade de tempo de armazenamento dos registros em dias
"""
DRF_AUDIT_TRAIL_RETENTION_PERIOD = getattr(
    settings, "DRF_AUDIT_TRAIL_RETENTION_PERIOD", 120
)

DRF_AUDIT_TRAIL_REQUEST_AUDIT_URLS = getattr(
    settings, "DRF_AUDIT_TRAIL_REQUEST_AUDIT_URLS", [r"^/api/.*?/"]
)
DRF_AUDIT_TRAIL_AUTH_URL = getattr(settings, "DRF_AUDIT_TRAIL_AUTH_URL", "/api/token/")
DRF_AUDIT_TRAIL_AUTH_STATUS_CODE_FALIED = getattr(
    settings, "DRF_AUDIT_TRAIL_AUTH_STATUS_CODE_FALIED", 401
)

DRF_AUDIT_TRAIL_DATABASE_ALIAS = getattr(
    settings, "DRF_AUDIT_TRAIL_DATABASE_ALIAS", "audit_trail"
)
DRF_AUDIT_TRAIL_USER_PK_NAME = getattr(settings, "DRF_AUDIT_TRAIL_USER_PK_NAME", "pk")



# Generated by Django 4.2.13 on 2024-05-22 01:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("drf_audit_trail", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="loginauditevent",
            name="request",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                to="drf_audit_trail.requestauditevent",
            ),
        ),
    ]



# Generated by Django 4.2.13 on 2024-05-21 21:41

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="RequestAuditEvent",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "extra_informations",
                    models.TextField(
                        blank=True, null=True, verbose_name="Extra information"
                    ),
                ),
                (
                    "datetime",
                    models.DateTimeField(auto_now_add=True, verbose_name="Datetime"),
                ),
                (
                    "ip_addresses",
                    models.CharField(
                        max_length=50, null=True, verbose_name="Ip addresses"
                    ),
                ),
                ("method", models.CharField(max_length=10, verbose_name="Method")),
                ("url", models.CharField(max_length=2048, verbose_name="URL")),
                (
                    "query_params",
                    models.CharField(
                        blank=True,
                        max_length=2048,
                        null=True,
                        verbose_name="Query Parameters",
                    ),
                ),
                (
                    "request_type",
                    models.CharField(
                        blank=True,
                        max_length=10,
                        null=True,
                        verbose_name="Request Type",
                    ),
                ),
                (
                    "status_code",
                    models.IntegerField(null=True, verbose_name="Status Code"),
                ),
                (
                    "response_time",
                    models.FloatField(null=True, verbose_name="Response Time (ms)"),
                ),
                (
                    "response_size",
                    models.IntegerField(
                        null=True, verbose_name="Response Size (bytes)"
                    ),
                ),
                (
                    "error_type",
                    models.CharField(
                        blank=True, max_length=255, null=True, verbose_name="Error Type"
                    ),
                ),
                (
                    "error_message",
                    models.TextField(
                        blank=True, null=True, verbose_name="Error Message"
                    ),
                ),
                (
                    "error_stacktrace",
                    models.TextField(
                        blank=True, null=True, verbose_name="Error Stacktrace"
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "verbose_name": "Request event",
                "verbose_name_plural": "Request events",
                "db_table": "request_audit_event",
            },
        ),
        migrations.CreateModel(
            name="LoginAuditEvent",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "extra_informations",
                    models.TextField(
                        blank=True, null=True, verbose_name="Extra information"
                    ),
                ),
                (
                    "datetime",
                    models.DateTimeField(auto_now_add=True, verbose_name="Datetime"),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("Sign in", "Sign in"),
                            ("Sign out", "Sign out"),
                            ("Failed", "Failed"),
                        ],
                        max_length=8,
                        verbose_name="Status",
                    ),
                ),
                (
                    "request",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="drf_audit_trail.requestauditevent",
                    ),
                ),
            ],
            options={
                "verbose_name": "Login audit event",
                "verbose_name_plural": "Login audit events",
                "db_table": "login_audit_event",
            },
        ),
    ]
